#ifndef CDDDB_INTERFACE_H
#define CDDB_INTERFACE_H

#include <fcntl.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "http.h"


#if defined(linux)

#include <linux/cdrom.h>
#define		cdte_track_address	cdte_addr.lba

#elif defined(sun) && defined(unix) && defined(__SVR4)

#include <sys/cdio.h>
#define CD_MSF_OFFSET	150
#define CD_FRAMES	75

#elif defined(__FreeBSD__)

#include <sys/cdio.h>
#define        CDROM_LBA       CD_LBA_FORMAT   /* first frame is 0 */
#define        CD_MSF_OFFSET   150     /* MSF offset of first frame */
#define        CD_FRAMES       75      /* per second */
#define        CDROM_LEADOUT   0xAA    /* leadout track */
#define        CDROMREADTOCHDR         CDIOREADTOCHEADER
#define        CDROMREADTOCENTRY       CDIOREADTOCENTRY
#define        cdrom_tochdr    ioc_toc_header
#define        cdth_trk0       starting_track
#define        cdth_trk1       ending_track
#define        cdrom_tocentry  ioc_read_toc_single_entry
#define        cdte_track      track
#define        cdte_format     address_format
#define        cdte_track_address	entry.addr.lba

#elif defined(__OpenBSD__) || defined(__NetBSD__)

#include <sys/cdio.h>
#define        CDROM_LBA       CD_LBA_FORMAT   /* first frame is 0 */
#define        CD_MSF_OFFSET   150     /* MSF offset of first frame */
#define        CD_FRAMES       75      /* per second */
#define        CDROM_LEADOUT   0xAA    /* leadout track */
#define        CDROMREADTOCHDR         CDIOREADTOCHEADER
#define        cdrom_tochdr    ioc_toc_header
#define        cdth_trk0       starting_track
#define        cdth_trk1       ending_track
#define        cdrom_tocentry  cd_toc_entry
#define        cdte_track      track
#define        cdte_track_address       addr.lba

#endif

#define CDDB_DEFAULT_SERVER "freedb.freedb.org"
#define BUF2SIZE (80*3)

/* from xmms/Input/cdaudio/cddb.h */
#define CDDB_MAX_PROTOCOL_LEVEL 3
#define CDDB_HOSTNAME_LEN 100
#define CDDB_LOG_MAX 100

typedef struct cddb_disc_header_t
{
    gchar *category;
    guint32 discid;
} cddb_disc_header_t;


typedef struct cddb_toc_entry
{
    long int discid;
    int numtrack;
    int *offsets;
    int length;
} cddb_toc_entry;

typedef struct trackinfo_t{
	gchar	*artist;
	gchar	*title;
	gint	num;
} trackinfo_t;

/*
 * For holding info on a complete CD.
 */
typedef struct cdinfo_t{
	gboolean is_valid;
        gint     numtrack;
	gchar	*albname;
	gchar	*artname;
	trackinfo_t tracks[100];
} cdinfo_t;


extern struct cddb_toc_entry *cddb_tocentry(char *devicename);
extern cdinfo_t    *cddb_lookup(char *devicename, char *cddbserver);

//long int cdcd_cd_stat (int cd_desc, struct disc_info *disc, GtkButton *button);
//void lookup_now (int cd_desc, struct disc_data *data, GtkButton *button);
//void cddb_lookup (int cd_desc, struct disc_data *data, GtkButton *button);
//int cddb_to_item_text(GtkButton *button, gpointer data);
//void nameartist (struct disc_data *dd);

#endif
