#include "msupport.h"
#include "minterface.h"

void paint_color(GtkWidget*);

/**********************************************************************
 *
 * These functions repaint the drawing area based on input from the spin
 * buttons
 *
 */
void
on_spb_changed(GtkEditable *editable, gpointer user_data)
{
}


void
on_spb_value_changed(GtkSpinButton *spinbutton, gpointer user_data)
{
    gchar *name;
    GtkWidget *da;

    name = (gchar*) gtk_widget_get_name( GTK_WIDGET(spinbutton) );
    
    if (!strcmp(name,"spinbutton1") ||
            !strcmp(name,"spinbutton3") ||
            !strcmp(name,"spinbutton4"))
        da = lookup_widget(GTK_WIDGET(spinbutton), "pcol_da");
    else if (!strcmp(name,"spinbutton5") ||
            !strcmp(name,"spinbutton6") ||
            !strcmp(name,"spinbutton7"))
        da = lookup_widget(GTK_WIDGET(spinbutton), "tcol_da");
    else if (!strcmp(name,"spinbutton8") ||
            !strcmp(name,"spinbutton9") ||
            !strcmp(name,"spinbutton10"))
        da = lookup_widget(GTK_WIDGET(spinbutton), "ccol_da");
    else if (!strcmp(name,"spinbutton11") ||
            !strcmp(name,"spinbutton12") ||
            !strcmp(name,"spinbutton13"))
        da = lookup_widget(GTK_WIDGET(spinbutton), "scol_da");
    gtk_widget_queue_draw_area(
            da,
            0,0,
            da->allocation.width,
            da->allocation.height);
    
}

/****************************************************************************
 *
 * Handles responses from the color selection dialog
 *
 */
void on_colorsel_ok_clicked(GtkButton *button, gpointer data)
{
    /* OK, so we're done selecting the color. Thus after getting the
     * color we should set the proper spin button values and also
     * repaint the corresponding drawing area */

    GtkWidget *da, *dlg;
    GdkColor color;

    dlg = lookup_widget( GTK_WIDGET(button) , "colorselectiondialog");

    da = g_object_get_data( G_OBJECT(dlg), "da");

    dlg = lookup_widget( GTK_WIDGET(button), "colorsel_innerdialog");
    gtk_color_selection_get_current_color(GTK_COLOR_SELECTION(dlg), &color);

    if (!strcmp(gtk_widget_get_name(da), "pcol_da"))
      {
        GtkWidget *sp1,*sp2,*sp3;

        sp1 = lookup_widget( GTK_WIDGET(da), "spinbutton1");
        sp2 = lookup_widget( GTK_WIDGET(da), "spinbutton3");
        sp3 = lookup_widget( GTK_WIDGET(da), "spinbutton4");

        gtk_spin_button_set_value( GTK_SPIN_BUTTON(sp1), (int)(color.red * 255 / 65535.0));
        gtk_spin_button_set_value( GTK_SPIN_BUTTON(sp2), (int)(color.green * 255 / 65535.0));
        gtk_spin_button_set_value( GTK_SPIN_BUTTON(sp3), (int)(color.blue * 255/ 65535.0));
      }
    else if (!strcmp(gtk_widget_get_name(da), "tcol_da"))
      {
        GtkWidget *sp1,*sp2,*sp3;

        sp1 = lookup_widget( GTK_WIDGET(da), "spinbutton5");
        sp2 = lookup_widget( GTK_WIDGET(da), "spinbutton6");
        sp3 = lookup_widget( GTK_WIDGET(da), "spinbutton7");

        gtk_spin_button_set_value( GTK_SPIN_BUTTON(sp1), (int)(color.red * 255 / 65535.0));
        gtk_spin_button_set_value( GTK_SPIN_BUTTON(sp2), (int)(color.green * 255 / 65535.0));
        gtk_spin_button_set_value( GTK_SPIN_BUTTON(sp3), (int)(color.blue * 255/ 65535.0));
      }
    else if (!strcmp(gtk_widget_get_name(da), "ccol_da"))
      {
        GtkWidget *sp1,*sp2,*sp3;

        sp1 = lookup_widget( GTK_WIDGET(da), "spinbutton8");
        sp2 = lookup_widget( GTK_WIDGET(da), "spinbutton9");
        sp3 = lookup_widget( GTK_WIDGET(da), "spinbutton10");

        gtk_spin_button_set_value( GTK_SPIN_BUTTON(sp1), (int)(color.red * 255 / 65535.0));
        gtk_spin_button_set_value( GTK_SPIN_BUTTON(sp2), (int)(color.green * 255 / 65535.0));
        gtk_spin_button_set_value( GTK_SPIN_BUTTON(sp3), (int)(color.blue * 255/ 65535.0));
      }
    else if (!strcmp(gtk_widget_get_name(da), "scol_da"))
      {
        GtkWidget *sp1,*sp2,*sp3;

        sp1 = lookup_widget( GTK_WIDGET(da), "spinbutton11");
        sp2 = lookup_widget( GTK_WIDGET(da), "spinbutton12");
        sp3 = lookup_widget( GTK_WIDGET(da), "spinbutton13");

        gtk_spin_button_set_value( GTK_SPIN_BUTTON(sp1), (int)(color.red * 255 / 65535.0));
        gtk_spin_button_set_value( GTK_SPIN_BUTTON(sp2), (int)(color.green * 255 / 65535.0));
        gtk_spin_button_set_value( GTK_SPIN_BUTTON(sp3), (int)(color.blue * 255/ 65535.0));
      }

    gtk_widget_queue_draw_area(
            da,
            0,0,
            da->allocation.width,
            da->allocation.height);

    gtk_widget_destroy(lookup_widget( GTK_WIDGET(button),"colorselectiondialog"));
}

void on_colorsel_cancel_clicked(GtkButton *button, gpointer data)
{
    GtkWidget *dlg;

    dlg = lookup_widget(GTK_WIDGET(button), "colorselectiondialog");
    if (dlg)
        gtk_widget_destroy(dlg);
    else
        g_warning("Could'nt get the color selection dialog window\n");
}

/*************************************************************************
 *
 * These are the callbacks for the drawing areas and all are handled by
 * common code in paint_color()
 *
 */

gboolean on_da_button_press_event(GtkWidget *w, GdkEventButton *e, gpointer data)
{
    GtkWidget *widget;
    GladeXML *xml;

    widget = lookup_widget( w, "window1" );
    xml = create_color_dialog(widget,w);

    return FALSE;
}

gboolean on_da_expose_event(GtkWidget *w, GdkEventExpose *e, gpointer data)
{
    paint_color(w);
    return FALSE;
}

gboolean on_da_configure_event(GtkWidget *w, GdkEventConfigure *e, gpointer data)
{
    paint_color(w);
    return FALSE;
}

void on_da_realize(GtkWidget *w, gpointer data)
{
    paint_color(w);
}

/********************************************************************
 *
 * These are the funcs that actually get the current color from the
 * spinbuttons and repaint the drawing areas
 *
 */

void get_spin_button_values(
        GtkWidget *w1, GtkWidget *w2, GtkWidget *w3,
        GdkColor *color)
{
    int r,g,b;

    r = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(w1));
    g = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(w2));
    b = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(w3));

    color->red = (int)(r * 65535 /255.0);
    color->green = (int)(g * 65535 / 255.0);
    color->blue = (int)(b * 65535 / 255.0);
}

void do_paint(GtkWidget *widget, GdkColor color)
{
    GdkGC *gc;
    GdkPixmap *pixmap;

    pixmap = gdk_pixmap_new(
            widget->window,
            widget->allocation.width,
            widget->allocation.height,
            -1);
    gc = gdk_gc_new(widget->window);
    gdk_gc_set_rgb_fg_color(gc,&color);
    gdk_draw_rectangle(pixmap,
            gc,
            TRUE, 0, 0,
            widget->allocation.width,
            widget->allocation.height);
    gdk_draw_drawable(
            widget->window,
            gc,
            pixmap,
            0,0,
            0,0,
            widget->allocation.width,
            widget->allocation.height);
}
    
                                                                                
    
void paint_color(GtkWidget *w)
{
    gchar *name;
    GtkWidget *top,*da,*sp1, *sp2, *sp3;
    GdkColor color;
    int r,g,b;

    name = (gchar*)gtk_widget_get_name(w);
    top = lookup_widget( w, "window1");

    if (!strcmp(name,"pcol_da"))
      {
        sp1 = lookup_widget( GTK_WIDGET(top), "spinbutton1");
        sp2 = lookup_widget( GTK_WIDGET(top), "spinbutton3");
        sp3 = lookup_widget( GTK_WIDGET(top), "spinbutton4");

        get_spin_button_values(sp1,sp2,sp3,&color);
        do_paint(w, color);
      }
    else if (!strcmp(name,"tcol_da"))
      {
        sp1 = lookup_widget( GTK_WIDGET(top), "spinbutton5");
        sp2 = lookup_widget( GTK_WIDGET(top), "spinbutton6");
        sp3 = lookup_widget( GTK_WIDGET(top), "spinbutton7");

        get_spin_button_values(sp1,sp2,sp3,&color);
        do_paint(w, color);
      }
    else if (!strcmp(name,"ccol_da"))
      {
        sp1 = lookup_widget( GTK_WIDGET(top), "spinbutton8");
        sp2 = lookup_widget( GTK_WIDGET(top), "spinbutton9");
        sp3 = lookup_widget( GTK_WIDGET(top), "spinbutton10");

        get_spin_button_values(sp1,sp2,sp3,&color);
        do_paint(w, color);
      }
    else if (!strcmp(name,"scol_da"))
      {
        sp1 = lookup_widget( GTK_WIDGET(top), "spinbutton11");
        sp2 = lookup_widget( GTK_WIDGET(top), "spinbutton12");
        sp3 = lookup_widget( GTK_WIDGET(top), "spinbutton13");

        get_spin_button_values(sp1,sp2,sp3,&color);
        do_paint(w, color);
      }
    
    
}
