#include <gtk/gtk.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "minterface.h"
#include "msupport.h"
#include "configparse.h"
#include "fileselect.h"

#define MAXLINE 1024

void
load_config                            (GtkWidget       *button)
{
    FILE *fd;
    gchar line[MAXLINE];
    gchar id[MAXLINE];
    gchar value[MAXLINE];
    gchar tmp2[MAXLINE];

    gchar cddbtmp[MAXLINE];
    GtkWidget *entry1;

    strcpy(tmp2, g_get_home_dir());
    strcat(tmp2, "/.gtkcdlabelrc");

    if ((fd = fopen(tmp2,"r")) == NULL) { 
        /* error opening file */
        g_print(_("Failed to open config file - using defaults\n"));
        return;
    }

    for (;;) {
        if (fgets(line,MAXLINE,fd) == NULL)
            break;

        /* skip empty or hashed lines */
        strip_string(line);
        if (*line == '#' || *line == '\0') 
            continue;

        /* parse lines */
        if (parse_config_line(line,id,value)) {
            g_print(_("Syntax error in config-file\n"));
        }	

        /* The color value settings need to be handled seperately */
        if (strstr(id,"spinbutton") || strstr(id,"linewidth_sp"))
          {
            gtk_spin_button_set_value( GTK_SPIN_BUTTON(lookup_widget(GTK_WIDGET(button),id)), atof(value));
            continue;
          }
            
        /* Get the page type */
        if (strstr(id,"papertype"))
          {
            gtk_option_menu_set_history(GTK_OPTION_MENU(lookup_widget(GTK_WIDGET(button),id)), atoi(value));
            continue;
          }

        if (strcmp( id, "save_on_exit\0") ) {
            entry1 = lookup_widget (GTK_WIDGET (button), id);
            gtk_entry_set_text(GTK_ENTRY(entry1),value);
        }
        else  
          {
            entry1 = lookup_widget (GTK_WIDGET (button), id);
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(entry1), atoi(value));
          }
    }

}




gint parse_config_line(gchar *iline, gchar *id, gchar *value) {
    gchar *p,*p2;
    gchar line[1024];
    gchar tmp[1024];

    strcpy(line,iline); 
    strcpy(id,"");
    p = strtok(line,"=");
    if (p != NULL) {
        /* got id */
        strcpy(id,p);
        strip_string(id);
    } else {
        return 1;
    }

    strcpy(tmp,"");
    p = strtok(NULL,"");
    if (p != NULL) {
        /* string after = */
        strcpy(tmp,p);
        strip_string(tmp);
    } else { 
        return 1;
    }

    /* now strip quotes from string */
    p = tmp;
    if (*p == '\"') {
        p2 = p+1;
    } else {
        p2 = p;
    }
    if (p[strlen(p)-1] == '\"') {
        p[strlen(p)-1] = '\0';
    }
    strcpy(value,p2);

    /* now reconvert escape-chars */
    escape_parse(value);

    /* all ok */
    return 0;
}

gchar *strip_string(gchar *str) {
    gint i,j;
    gint c1;

    if ( str == NULL) return (NULL);

    /* count how many leading chars to be whitespace */
    for(i=0; i<strlen(str); i++) {
        if (str[i] != ' ' && str[i] != '\t' && str[i] != '\r') 
            break;
    }

    /* count how many trailing chars to be whitespace */
    for(j=strlen(str)-1; j >= 0; j--) {
        if (str[j] != ' ' && str[j] != '\t' && str[j] != '\n') 
            break;
    }

    /* string contains only whitespace? */
    if (j<i) {
        str[0] = '\0';
        return(str);
    }

    /* now move the chars to the front */
    for(c1=i; c1 <= j; c1++) {
        str[c1-i] = str[c1]; 
    }
    str[j+1-i] = '\0';      

    return(str);
}

/* parse escape-chars in string -> e.g. translate \n to newline */

gchar *escape_parse(gchar *str) {
    gchar tmp[MAXLINE];
    gchar c;
    gint i,j;

    if ( str == NULL) return (NULL);

    j = 0;
    for(i=0; i<strlen(str); i++) {
        c = str[i];
        if (c == '\\') {
            i++;
            switch(str[i]) {

                case 'n':
                    c = '\n';
                    break;

                case 't':
                    c = '\t';
                    break;

                case 'b':
                    c = '\b';
                    break;

                default:
                    c = str[i];
            }
        }	

        tmp[j]=c;
        j++;
    }

    tmp[j] = '\0';

    strcpy(str,tmp);
    return(str);
}


