
#include "filefunc.h"


void getdirs(char *cwd, int *numdir, GArray **dirlist)
{
    char *item;
    int i;

    DIR *dp;
    struct dirent *d;
    struct stat statbuf;

    dp = opendir(cwd);
    if (dp != NULL)
      {
        while (d = readdir(dp))
          {
            char *t;

            if (!strcmp(d->d_name,".") || !strcmp(d->d_name,"..")) continue;

            t = (char*)malloc(sizeof(char*) * (strlen(cwd)+strlen(d->d_name)+2));
            sprintf(t,"%s/%s",cwd,d->d_name);
            stat(t,&statbuf);
            
            if (S_ISDIR(statbuf.st_mode))
              {
                (*numdir)++;
                g_array_append_val(*dirlist,t);
                getdirs(t, numdir, dirlist);
              }
          }
      }
    else 
      {
        *numdir = 0;
        return;
      }
    closedir(dp);

    return;
}
                
                
int check_filename(char *filename, int outputfilecheck, char *msg)
{
    struct stat statbuf;
    char *errstr = NULL;
    char *e;
    int status, flag = 0;
    FILE *f;

    if (strlen(filename) == 0)
      {
        GtkWidget *w;
        GladeXML *xml = create_error_dialog();

        if (msg)
          {
            errstr = malloc(sizeof(char) * (strlen(msg) + strlen("No filename was specified") + 3));
            sprintf(errstr,"%s: %s", msg, "No file name was specified");
          }
        else
          {
            errstr = malloc(sizeof(char) * (strlen("No filename was specified") + 3));
            sprintf(errstr,"%s", "No file name was specified");
          }

        w = glade_xml_get_widget(xml, "error_dialog_label");
        gtk_label_set_text(GTK_LABEL(w), errstr);

        free(errstr);
        return(-1);
      }

    switch(outputfilecheck)
      {
        case CHECK_FILE_BLOCK:
            status = stat(filename, &statbuf);
            if (status == -1 && errno == ENOENT)
              {
                /* file probably does'nt exist */
                e = strerror(errno);
                if (msg)
                  {
                    errstr = malloc(sizeof(char) * (strlen(msg) + strlen(e) + strlen(filename) + 8));
                    sprintf(errstr,"%s :: %s : %s", msg, filename, e);
                  }
                else
                  {
                    errstr = malloc(sizeof(char) * (strlen(e) + strlen(filename) + 8));
                    sprintf(errstr,"%s : %s",filename, e);        
                  }
              }
            else if (!S_ISBLK(statbuf.st_mode))
              {
                /* report the error */
                e = strdup("Does'nt seem to be a block device");
                if (msg)
                  {
                    errstr = malloc(sizeof(char) * (strlen(msg) + strlen(e) + strlen(filename) + 8));
                    sprintf(errstr,"%s :: %s : %s", msg, filename, e);
                  }
                else
                  {
                    errstr = malloc(sizeof(char) * (strlen(e) + strlen(filename) + 8));
                    sprintf(errstr,"%s : %s",filename, e);        
                  }
              }
            break;
            
        case CHECK_FILE_VALID:
            status = stat(filename, &statbuf);
            if (status == -1 && errno == ENOENT)
              {
                /* file probably does'nt exist */
                e = strerror(errno);
                if (msg)
                  {
                    errstr = malloc(sizeof(char) * (strlen(msg) + strlen(e) + strlen(filename) + 8));
                    sprintf(errstr,"%s :: %s : %s", msg, filename, e);
                  }
                else
                  {
                    errstr = malloc(sizeof(char) * (strlen(e) + strlen(filename) + 8));
                    sprintf(errstr,"%s : %s",filename, e);        
                  }
              }
            else if (!S_ISREG(statbuf.st_mode))
              {
                /* report the error */
                e = strdup("Does'nt seem to be a regular file");
                if (msg)
                  {
                    errstr = malloc(sizeof(char) * (strlen(msg) + strlen(e) + strlen(filename) + 8));
                    sprintf(errstr,"%s :: %s : %s", msg, filename, e);
                  }
                else
                  {
                    errstr = malloc(sizeof(char) * (strlen(e) + strlen(filename) + 8));
                    sprintf(errstr,"%s : %s",filename, e);        
                  }
              }
            break;

        case CHECK_FILE_OUTPUT:
            status = stat(filename, &statbuf);
            if (status == -1 && errno == ENOENT) flag = 1;
            f = fopen(filename,"a");
            if (!f)
              {
                e = strerror(errno);
                if (msg)
                  {
                    errstr = malloc(sizeof(char) * (strlen(msg) + strlen(e) + strlen(filename) + 8));
                    sprintf(errstr,"%s :: %s : %s", msg, filename, e);
                  }
                else
                  {
                    errstr = malloc(sizeof(char) * (strlen(e) + strlen(filename) + 8));
                    sprintf(errstr,"%s : %s",filename, e);        
                  }
              }
            else 
              {
                fclose(f);
                if (flag == 1) unlink(filename);
                return(0);
              }
            break;
      }

    if (errstr)
      {
        GtkWidget *w;
        GladeXML *xml = create_error_dialog();

        w = glade_xml_get_widget(xml, "error_dialog_label");
        gtk_label_set_text(GTK_LABEL(w), errstr);

        free(errstr);
        return(-1);
      }
    else return(0);
}
               
int check_graphics(GtkWidget *anchor, char **filename)
{
    /* return 1 - if convert was sucessful, and input string contains
     * new name. The caller should remember to delete the file
     * return 0 - if it was an EPS and no conversion necessary
     * return -1 - any error. Print all error messages here
     */

    /* we assume that on entry the filename points to a valid file that
     * exists and can be read */
    int i,j,status;
    char *error_buffer;
    char buffer[100];
    char *cmd;
    char *convertprog, *line;
    char t[] = "gcdlciXXXXXX";
    char *tmpfile;

    GtkWidget *w;
    FILE *in, *fp;


    /* first see if the supplied file is an EPS file. If so return */
    fp = fopen(*filename,"r");
    i = 0; line = NULL;
    getline(&line,&i,fp);
    if (strstr(line,"%!PS-Adobe") && strstr(line,"EPS"))
      {
        fclose(fp);
        return(0);
      }


    /* OK so it was'nt an EPS so we need to try and convert */

    w = lookup_widget(anchor,"convert_entry");
    convertprog = (char*)gtk_entry_get_text( GTK_ENTRY(w) );
    convertprog = g_filename_from_utf8(convertprog,-1,NULL,NULL,NULL);

    /* check we have a value for convert */
    if (strlen(convertprog) == 0)
      {
        GtkWidget *w;
        GladeXML *xml = create_error_dialog();
        w = glade_xml_get_widget(xml, "error_dialog_label");
        gtk_label_set_text(GTK_LABEL(w), 
                "You have not specified the program with which to convert images to EPS format\n"
                "If you have the ImageMagick suite of programs then specify the path to the program convert");
        return( -1);
      }

    /* check that convert is a valid file */
    status = check_filename(convertprog,CHECK_FILE_VALID,"The path to 'convert'");
    if (status == -1) return(-1);

    // This construct has the same securoty problem as mktemp() but it
    // gets rid of the compiler warning
    i = mkstemp(t);
    close(i);
    unlink(t);

    tmpfile = malloc(sizeof(char) * (strlen(t) + strlen(".eps") + 2));
    sprintf(tmpfile,"%s.eps",t);

    cmd = malloc(sizeof(char) * (strlen("2>/dev/null") + strlen(convertprog) + strlen(*filename) + strlen(tmpfile) + 6));
    sprintf(cmd,"%s %s %s 2>/dev/null", convertprog, *filename, tmpfile);
            
    error_buffer = malloc(sizeof(char) * MAXLINE);
    error_buffer[0] = '\0';
    in = popen(cmd, "r");
    while (fgets(buffer, 100, in) != NULL)
      {
        if (strlen(error_buffer) + strlen(buffer) > MAXLINE)
          {
            error_buffer = realloc(error_buffer, sizeof(char) * (strlen(error_buffer) + strlen(buffer) + 1));
            error_buffer[ strlen(error_buffer) - 1] = '\0';
          }
        strcat(error_buffer , buffer);
      }
    fclose(in);

    /* show the error we got */
    if (strlen(error_buffer) != 0)
      {
        GladeXML *xml;
        GtkWidget *w;
        GtkTextBuffer *tbuffer;
        
        xml = create_cdlabelgen_error();
        w = glade_xml_get_widget(xml, "ced_textview");

        tbuffer = gtk_text_view_get_buffer( GTK_TEXT_VIEW(w) );
        gtk_text_buffer_set_text(tbuffer, error_buffer, strlen(error_buffer));
        return (-1);
      }

    *filename = strdup(tmpfile);
    
    free(tmpfile);
    free(cmd);

    return( 1);
    

}

