#include <gtk/gtk.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/stat.h> 
#include <dirent.h>

#include "fileselect.h"
#include "mcallbacks.h"
#include "minterface.h"
#include "msupport.h"

#define MAXLINE 1024


char *my_basename(const char *filename);
void wait_and_process_events();
gint dialog_done3;
void file_selector_ok(GtkWidget *widget, GtkFileSelection *fs);
gint dialog_delete_event3(GtkWidget *widget, GdkEvent *event, gpointer data);

gint dialog_delete_event3(GtkWidget *widget, GdkEvent *event, gpointer data)
{

	dialog_done3 = -1;
	return(TRUE);
}
		

/* fileselector-box ok-button callback */

void file_selector_ok(GtkWidget *widget, GtkFileSelection *fs) {

	dialog_done3 = 1;
}


/* display a file-selector box with defname preselected. Return "" 
   or filename in retvalue */

void show_file_selector(gchar *defname, gchar *retvalue) {
GtkWidget *dialog;

	/* if another dialog is running, ignore */
	if (dialog_done3 == 999) {
		return;
	}


	/* mark our dialog as running */
	dialog_done3 = 999;

	/* create new window and position it relative to the main window */
	dialog = gtk_file_selection_new (_("Select File"));

	/* set default selection */
	if (defname != NULL)
		gtk_file_selection_set_filename(GTK_FILE_SELECTION(dialog),defname);
				
        gtk_signal_connect (GTK_OBJECT (dialog), "delete_event",
                GTK_SIGNAL_FUNC (dialog_delete_event3), dialog);

        gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION(dialog)->ok_button),
		"clicked", GTK_SIGNAL_FUNC(file_selector_ok), dialog);

        gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION(dialog)->cancel_button),
		"clicked", GTK_SIGNAL_FUNC(dialog_delete_event3), dialog);
	
	gtk_widget_show(dialog);


	/* now wait until button is pressed */
	while (dialog_done3 == 999) {
		wait_and_process_events();
	}

        gtk_grab_remove(GTK_WIDGET(dialog));

	if (dialog_done3 != -1) {
		strcpy(retvalue,gtk_file_selection_get_filename (GTK_FILE_SELECTION (dialog)));
	} else {
		/* cancel or delete_event */
		strcpy(retvalue,"\0");
	}	

	/* remove dialog window */
        gtk_widget_destroy(dialog);
}


void wait_and_process_events() 
{
    while (gtk_events_pending())
        gtk_main_iteration();
    usleep(100);
}

gint is_file(gchar *path)
{
    struct stat buf;

    if (stat(path,&buf) != 0)
      {
        return 0;
      }
    return 1;   
}

/* return 1 if path is a directory and 0 if not or invalid path */

gint is_dir(gchar *path) {
struct stat buf;

	if (stat(path,&buf) != 0) {
		return 0;
	}

	if (S_ISDIR(buf.st_mode) == 1) {
		return 1;
	} else {
		return 0;
	}
}

/* takes the data from a dir and puts it into item_text_col after emptying it */

void dir_to_item_text (GtkButton *button, gpointer data) 
{
    char path[MAXLINE];
    struct stat buf;
    DIR *dir;
    struct dirent *dirp;
    GtkWidget *item_text_col;
    GtkTextBuffer *buffer;
    GtkTextIter start,end,curr;

    item_text_col	 =	lookup_widget (GTK_WIDGET (button), data);
    printf("%s\n", gtk_widget_get_name(item_text_col));

    show_file_selector("/mnt/cdrom/",path);

    buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(item_text_col));
    //gtk_text_buffer_get_bounds(buffer,&start,&end);
    //gtk_text_buffer_delete(buffer,&start,&end);

    if (-1 == (stat(path, &buf))) {
        return;
    }

    //	gtk_text_freeze(GTK_TEXT(item_text_col));

    if (S_ISDIR(buf.st_mode)) {
        if (NULL == (dir = opendir(path))) {
            return;
        }
        /* for every filename in path, add it to item_list */
        while (NULL != (dirp = readdir(dir))) {
            if (0 == (strcmp(dirp->d_name, "."))
                    || 0 == strcmp(dirp->d_name, ".."))
                continue;
            gtk_text_buffer_get_end_iter(buffer,&end);
            gtk_text_buffer_insert(buffer,&end,dirp->d_name,-1);
            gtk_text_buffer_get_end_iter(buffer,&end);
            gtk_text_buffer_insert(buffer,&end,"\n",1);
        }
        if (closedir(dir) < 0)
            g_warning("cdlabel.c::dir_to_item_text: close() failed");
    } 
    else /* just one filename, not sure it makes sense, but... */
      {
        gtk_text_buffer_get_end_iter(buffer,&end);
        gtk_text_buffer_insert(buffer, &end, my_basename(path), -1);
        gtk_text_buffer_get_end_iter(buffer,&end);
        gtk_text_buffer_insert(buffer, &end, "\n", 1);
      }
    gtk_text_view_set_buffer(GTK_TEXT_VIEW(item_text_col), buffer);

    //gtk_text_thaw(GTK_TEXT(item_text_col));

}

char *my_basename(const char *filename)
{
	char *p = strrchr(filename, '/');
	return p ? p + 1 : (char *)filename;
}

int save_file (GtkButton *button, char *filename)
{
	FILE *file;
	long int length;
	char *cptr;
	GtkWidget *text1;
        GtkTextBuffer *buffer;
        GtkTextIter start,end;
        
	text1	 =	lookup_widget (GTK_WIDGET (button), "text1");
        buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text1));
        gtk_text_buffer_get_bounds(buffer, &start, &end);
	
	if ((file = fopen(filename, "w")) == NULL)
		return 1;
	
	//length = gtk_text_get_length(GTK_TEXT(text1));
        length = gtk_text_buffer_get_char_count(buffer);
        
	if (length==0) return 1;
	
	//cptr = gtk_editable_get_chars(GTK_EDITABLE(text1),0,length);
        cptr = gtk_text_buffer_get_text(buffer, &start, &end, 0);
        cptr = g_locale_from_utf8(cptr, g_utf8_strlen(cptr,-1), NULL, NULL, NULL);
	
	if (cptr)
		fwrite(cptr, strlen(cptr), 1, file);
		
	fclose(file);
	return 0;
}

  /* Load the file text.c into the text window */
 
void file_to_item_text (GtkButton *button) 
{
    char path[MAXLINE];
    FILE *infile;
    GtkWidget *file_text_col;
    GtkTextBuffer *tbuffer;
    GtkTextIter start,end;

    file_text_col	 =	lookup_widget (GTK_WIDGET (button), "text1");
    tbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(file_text_col));

    show_file_selector(NULL ,path);
    infile = fopen( path, "r");

    if (infile)
      {
        char buffer[MAXLINE];
        int nchars;
        gchar *utf8;
        while (1)
          {
            nchars = fread(buffer, 1, 1024, infile);
            utf8 = g_locale_to_utf8(buffer, nchars, NULL, NULL, NULL);
            if (!utf8)
              {
                GladeXML *xml;
                GtkWidget *w;
                char *errstr;
                xml = create_error_dialog();
                errstr = strdup("Seems that you need to set your LOCALE properly,\nsince the text file does not match your current locale.");
                w = glade_xml_get_widget(xml, "error_dialog_label");
                gtk_label_set_text(GTK_LABEL(w), errstr);
                return;
              }
            gtk_text_buffer_get_end_iter(tbuffer, &end);
            gtk_text_buffer_insert(tbuffer, &end, utf8, strlen(utf8));

            if (nchars < 1024)
                break;
          }
        fclose (infile);
      }
    gtk_text_view_set_buffer(GTK_TEXT_VIEW(file_text_col), tbuffer);
}

