#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <sys/stat.h>
#include <unistd.h>
#include <locale.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <glob.h>

#include "localefunc.h"
#include "mcallbacks.h"
#include "minterface.h"
#include "msupport.h"
#include "cddb.h"
#include "id3.h"
#include "filefunc.h"



int run_cdlabelgen(GtkButton *button, gpointer user_data);

void sensitize(GtkWidget *anchor, int num, char **names, gboolean state)
{
    int i;
    GtkWidget *w;

    for (i = 0; i < num; i++)
      {
        w = lookup_widget( anchor, names[i]);
        gtk_widget_set_sensitive(w,state);
      }
}

void get_rgb_values(GtkButton *x, char *sr, char *sg, char *sb, int *r, int *g, int *b)
{
    GtkWidget *w = lookup_widget(GTK_WIDGET(x), sr);
    *r = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON(w) );

    w = lookup_widget(GTK_WIDGET(x),sg);
    *g = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON(w) );
    
    w = lookup_widget(GTK_WIDGET(x),sb);
    *b = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON(w) );
}


void syntaxerror (GtkButton *button)
{
    GtkWidget *label;
//    label			=	lookup_widget (GTK_WIDGET (button), "status_label");
//    gtk_label_set_text(GTK_LABEL(label),_("FATAL ERROR - cdlabelgen exited abnormally"));
}


gint remove_illegal_chars(gchar *str)
{
    gint i,j;

    if (str == NULL) return 0;

    j = 0;
    for (i = 0; i < strlen(str); i++) {
        switch ((int)str[i]) {
            case 34:
            case 96:
                str[i] = '\'';
                j = 1;
                break;
        }
    }
    return (j);
}

int
run_cdlabelgen (GtkButton *button, gpointer  user_data )
{
    /* error messages stuff */
    int status, graphic_status1, graphic_status2;

    FILE *in, *popen ();
    char buffer[100], tmp[128];
    //char error_buffer[MAXLINE]="\0";
    char *error_buffer;

    char cdlabel_add[MAXLINE]="\0";
    gchar cdlabeltxt[MAXLINE];
    
    GtkTextBuffer *tbuffer;
    GtkTextIter startiter, enditer;


    gchar *kategori;
    gchar *underkategori;
    gchar *index;
    gchar *command;
    gchar *where_is_cdlabelgen;
    gchar  *whichtemplate;

    gchar *eps_filename;
    gchar *tray_filename;
    gchar *cover_items_value;
    gchar *eps_scale;
    gchar *tray_scale;
    gchar *date_entry;

    gchar *volume_info;

    GtkWidget *volume_entry;
    GtkWidget *template_entry;
    GtkWidget *entry_eps_filename;
    GtkWidget *entry_tray_filename;
    GtkWidget *entry_eps_scale;
    GtkWidget *entry_tray_scale;
    GtkWidget *entry_date_entry;
    GtkWidget *use_cover_items;
    GtkWidget *entry_cover_items;

    GtkWidget *entry_kategori;   
    GtkWidget *entry_underkategory;
    GtkWidget *text1;
    GtkWidget *cdlabelgen_entry;

    GtkWidget *use_eps;
    GtkWidget *use_tray;
    GtkWidget *no_date_radio;
    GtkWidget *word_wrap;
    GtkWidget *no_plaque;
    GtkWidget *no_cover_plaque;
    GtkWidget *clipping;
    GtkWidget *label;
    
    GtkWidget *tmpwidget;
    int r,g,b;
    float linewidth;

    GtkWidget *papertype;
    int papertype_option = 0; // default is letter option
    float papertype_offset = 0.8; 

    snprintf(cdlabeltxt, MAXLINE-1,"%s/gtkcdlabel-tmp-txt", getenv("HOME"));

    /* Get line width */
    linewidth = gtk_spin_button_get_value_as_float(
            GTK_SPIN_BUTTON( lookup_widget(GTK_WIDGET(button), "linewidth_sp") ));
    sprintf(tmp," -l %s ", check_locale_float(linewidth));
    strcat(cdlabel_add, tmp);

    /* Get the various colors */
    get_rgb_values(
            button,
            "spinbutton1",
            "spinbutton3",
            "spinbutton4",
            &r,&g,&b);
    sprintf(tmp," --plaque-color %d,%d,%d ", r,g,b);
    strcat(cdlabel_add, tmp);

    get_rgb_values(
            button,
            "spinbutton5",
            "spinbutton6",
            "spinbutton7",
            &r,&g,&b);
    sprintf(tmp," --text-color %d,%d,%d ", r,g,b);
    strcat(cdlabel_add, tmp);

    get_rgb_values(
            button,
            "spinbutton8",
            "spinbutton9",
            "spinbutton10",
            &r,&g,&b);
    sprintf(tmp," --category-color %d,%d,%d ", r,g,b);
    strcat(cdlabel_add, tmp);

    get_rgb_values(
            button,
            "spinbutton11",
            "spinbutton12",
            "spinbutton13",
            &r,&g,&b);
    sprintf(tmp," --subcategory-color %d,%d,%d ", r,g,b);
    strcat(cdlabel_add, tmp);

    
    /* Get the paper type */
    papertype = lookup_widget( GTK_WIDGET(button), "papertype_optionmenu");
    papertype_option = gtk_option_menu_get_history( GTK_OPTION_MENU(papertype) );
    switch( papertype_option )
      {
        case 2: // A4 envelope
            papertype_offset = 1.1; 
            break;
        case 1: // A4
            papertype_offset = 1.5; 
            break;
        case 0: // letter
            papertype_offset = 0.8;
            break;
      }
    sprintf(tmp," -y %s ", check_locale_float(papertype_offset));
    strcat(cdlabel_add, tmp);
    

    /* get the remaining UI element settings */
    entry_eps_filename      =       lookup_widget (GTK_WIDGET (button), "entry_eps_filename");
    entry_tray_filename     =       lookup_widget (GTK_WIDGET (button), "entry_tray_filename");
    entry_eps_scale         =       lookup_widget (GTK_WIDGET (button), "eps_scale");
    entry_tray_scale        =       lookup_widget (GTK_WIDGET (button), "tray_scale");
    entry_date_entry        =       lookup_widget (GTK_WIDGET (button), "date_entry");
    use_cover_items		=       lookup_widget (GTK_WIDGET (button), "use_cover_items");
    entry_cover_items	=	lookup_widget (GTK_WIDGET (button), "entry_cover_items");

    entry_kategori          =       lookup_widget (GTK_WIDGET (button), "entry_kategori");
    entry_underkategory     =       lookup_widget (GTK_WIDGET (button), "entry_underkategory");
    text1                   =       lookup_widget (GTK_WIDGET (button), "text1");
    cdlabelgen_entry        =       lookup_widget (GTK_WIDGET (button), "cdlabelgen_entry");
    template_entry = lookup_widget(GTK_WIDGET(button), "template_entry");
    volume_entry = lookup_widget(GTK_WIDGET(button), "volume_entry");

    use_eps                 =       lookup_widget (GTK_WIDGET (button), "use_eps");
    use_tray                =       lookup_widget (GTK_WIDGET (button), "use_tray");
    no_date_radio           =       lookup_widget (GTK_WIDGET (button), "no_date_radio");
    word_wrap               =       lookup_widget (GTK_WIDGET (button), "word_wrap");
    no_plaque               =       lookup_widget (GTK_WIDGET (button), "no_plaque");
    no_cover_plaque         =       lookup_widget (GTK_WIDGET (button), "no_cover_plaque");
    clipping                =       lookup_widget (GTK_WIDGET (button), "clipping");
    //	slimcase_button         =       lookup_widget (GTK_WIDGET (button), "slimcase_button");

    kategori                =       (gchar*)get_entry_text_in_locale( entry_kategori );
    underkategori           =       (gchar*)get_entry_text_in_locale( entry_underkategory );
    where_is_cdlabelgen     =       (gchar*)get_entry_text_in_locale( cdlabelgen_entry );                                        
    whichtemplate = (gchar*)get_entry_text_in_locale(template_entry);
    date_entry              =       (gchar*)get_entry_text_in_locale( entry_date_entry );
    eps_filename            =       (gchar*)get_entry_text_in_locale( entry_eps_filename );
    tray_filename = (gchar*)get_entry_text_in_locale( entry_tray_filename );
    volume_info = (gchar*)get_entry_text_in_locale( volume_entry );


    /* make sure the number values are valid numbers */
    eps_scale = check_locale((gchar*)get_entry_text_in_locale( entry_eps_scale ));
    tray_scale = check_locale((gchar*)get_entry_text_in_locale( entry_tray_scale ));
    cover_items_value =  check_locale((gchar*)get_entry_text_in_locale( entry_cover_items ));

    /* Get the index text */
    tbuffer = gtk_text_view_get_buffer( GTK_TEXT_VIEW(text1) );
    gtk_text_buffer_get_bounds(tbuffer, &startiter, &enditer);
    index = g_locale_from_utf8( gtk_text_buffer_get_text(tbuffer, &startiter, &enditer, FALSE), -1, NULL, NULL, NULL);

    /* check that the path to cdlabelgen is a valid file */
    status = check_filename(where_is_cdlabelgen,CHECK_FILE_VALID,"The path to cdlabelgen");
    if (status < 0) return(-1);

    /* check that the template file specified is also valid */
    status = check_filename(whichtemplate,CHECK_FILE_VALID,"The path to the template file");
    if (status < 0) return(-1);

    if (!save_file(button,cdlabeltxt))
      {
        strcat(cdlabel_add , " -f \"\0");
        strcat(cdlabel_add , cdlabeltxt);
        strcat(cdlabel_add , "\"\0");
      } 	 	 	 	 	 	 	            

    if (GTK_TOGGLE_BUTTON(use_eps)->active)
      {
        /* check that we have a valid filename */
        status = check_filename(eps_filename,CHECK_FILE_VALID,"Cover image");
        if (status < 0) return(status);
        if ( strlen(eps_filename) != 0)
          {
            float x,y;

            graphic_status1 = check_graphics(GTK_WIDGET(button),&eps_filename);
            if (graphic_status1 == -1) return(-1);

            strcat(cdlabel_add , " -e \"\0");
            strcat(cdlabel_add , eps_filename);
            strcat(cdlabel_add , "\"\0");

            if (GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(button),"cover_bg_choice"))->active)
                // if we use the image as background we can forget about
                // scale and offset values
                strcat(cdlabel_add," -S 0 \0");
            else
              {
                if ( strlen(eps_scale) != 0)
                  {
                    strcat(cdlabel_add , " -S \0");
                    strcat(cdlabel_add , eps_scale);
                  }

                /* add the offsets */
                x = gtk_spin_button_get_value_as_float(
                        GTK_SPIN_BUTTON( lookup_widget(GTK_WIDGET(button), "coverxoffset") ));
                y = gtk_spin_button_get_value_as_float(
                        GTK_SPIN_BUTTON( lookup_widget(GTK_WIDGET(button), "coveryoffset") ));
                sprintf(tmp,",%s,%s ",check_locale_float(x),check_locale_float(y));
                strcat(cdlabel_add,tmp);
              }

          }
      }

    if (GTK_TOGGLE_BUTTON(use_tray)->active)
      {
        /* check that we have a valid filename */
        status = check_filename(tray_filename,CHECK_FILE_VALID,"Tray image");
        if (status < 0) return(status);
        if ( strlen(tray_filename) != 0)
          {
            float x,y;

            graphic_status2 = check_graphics(GTK_WIDGET(button),&tray_filename);
            if (graphic_status2 == -1) return(-1);
            
            strcat(cdlabel_add , " -E \"\0");
            strcat(cdlabel_add , tray_filename);
            strcat(cdlabel_add , "\"\0");

            if (GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(button),"tray_bg_choice"))->active)
              {
                /* see what type of background we want */
                switch(gtk_option_menu_get_history( GTK_OPTION_MENU(lookup_widget(GTK_WIDGET(button),"tray_bg_option")) ))
                  {
                    case 0:
                        strcat(cdlabel_add, " -T fill1 \0");
                        break;
                    case 1:
                        strcat(cdlabel_add, " -T fill2 \0");
                        break;
                  }
              }
            else
              {
                if ( strlen(tray_scale) != 0)
                  {
                    strcat(cdlabel_add , " -T \0");
                    strcat(cdlabel_add , tray_scale);
                  }

                /* add the offsets */
                x = gtk_spin_button_get_value_as_float(
                        GTK_SPIN_BUTTON( lookup_widget(GTK_WIDGET(button), "trayxoffset") ));
                y = gtk_spin_button_get_value_as_float(
                        GTK_SPIN_BUTTON( lookup_widget(GTK_WIDGET(button), "trayyoffset") ));
                sprintf(tmp,",%s,%s ",check_locale_float(x),check_locale_float(y));
                strcat(cdlabel_add,tmp);
              }


          }
      }

    if (GTK_TOGGLE_BUTTON(use_cover_items)->active)
      {
        if ( strlen(cover_items_value) != 0)
          {
            strcat(cdlabel_add , " -v \0");
            strcat(cdlabel_add , cover_items_value);
          }
      }

    if (GTK_TOGGLE_BUTTON(no_plaque)->active)
        strcat(cdlabel_add, " -b\0");

    if (GTK_TOGGLE_BUTTON(no_cover_plaque)->active)
        strcat(cdlabel_add, " -C\0");

    /* Get the insert style */
    tmpwidget = lookup_widget( GTK_WIDGET(button), "style_optionmenu");
    switch(gtk_option_menu_get_history(GTK_OPTION_MENU(tmpwidget)))
      {
        case 0:
            break;
        case 1:
            strcat(cdlabel_add , " -m \0");
            break;
        case 2:
            strcat(cdlabel_add , " -M \0");
            break;
      }
            
    /* If volume info was supplied add the CLI switch */
    if (strlen(volume_info) != 0)
      {
        strcat(cdlabel_add, " -n \"\0");
        strcat(cdlabel_add, volume_info);
        strcat(cdlabel_add , "\"\0");
      }

    /* Word wrap or clipping, NOT both! */
    if (GTK_TOGGLE_BUTTON(word_wrap)->active)
        strcat(cdlabel_add, " -w\0");
    else
        strcat(cdlabel_add, " -p\0");


    if (GTK_TOGGLE_BUTTON(no_date_radio)->active)
      {
        strcat(cdlabel_add, " -D\0");
      }
    else
      {
        if ( strlen(date_entry) != 0)
          {
            remove_illegal_chars(date_entry);
            strcat(cdlabel_add, " -d \"\0");
            strcat(cdlabel_add , date_entry);
            strcat(cdlabel_add , "\"\0");
          }
      }

    remove_illegal_chars(kategori);
    remove_illegal_chars(underkategori);

    command = g_malloc(strlen(where_is_cdlabelgen) + strlen(kategori) + strlen(underkategori) + strlen(cdlabel_add) + strlen(cdlabeltxt) + strlen(whichtemplate) + 40);  // 40

    sprintf(command, "%s -c \"%s\"  -s \"%s\" -o %s/gtkcdlabel-tmp-ps -t %s %s 2>&1", where_is_cdlabelgen, kategori, underkategori ,getenv("HOME") ,whichtemplate, cdlabel_add);
    g_print(_("cdlabelgen command line: %s\n"),command);

    error_buffer = malloc(sizeof(char) * MAXLINE);
    error_buffer[0] = '\0';
    in = popen(command, "r");
    while (fgets(buffer, 100, in) != NULL)
      {
        if (strlen(error_buffer) + strlen(buffer) > MAXLINE)
          {
            error_buffer = realloc(error_buffer, sizeof(char) * (strlen(error_buffer) + strlen(buffer) + 1));
            error_buffer[ strlen(error_buffer) - 1] = '\0';
          }
        strcat(error_buffer , buffer);
      }

    /* show the error we got */
    if (strlen(error_buffer) != 0)
      {
        GladeXML *xml;
        GtkWidget *w;
        GtkTextBuffer *tbuffer;
        
        xml = create_cdlabelgen_error();
        w = glade_xml_get_widget(xml, "ced_textview");

        tbuffer = gtk_text_view_get_buffer( GTK_TEXT_VIEW(w) );
        gtk_text_buffer_set_text(tbuffer, error_buffer, strlen(error_buffer));
      }
    remove(cdlabeltxt);
    fclose(in);

    /* remove the converted graphics files if present */
    if (graphic_status1 == 1)
        unlink(eps_filename);
    if (graphic_status2 == 1)
        unlink(tray_filename);
}

void
on_get_dir_clicked                     (GtkButton       *button,
        gpointer         user_data)
{
    dir_to_item_text(button,"text1");
}


void
on_get_file_clicked(GtkButton *button, gpointer user_data)
{
    char tmp[MAXLINE];
    struct stat statbuf;

    char *name;
    gchar *startdir;
    GtkWidget *entrywidget;

    /* If the button was the Browse buttons for the EPS cover and tray
     * images then we should pull the value from the Stock Images entry
     * box and use that as the start dir for the file selection box. So
     * to check whether the button was one of these two we try and get
     * the data stored in the widget under the key value "varname". If
     * it is present it we can check for the corresponding Glade
     * variable (or whatever was stored when the widget was created in
     * interface.c) otherwise it iwll be NULL and it is not the button
     * we want*/
    entrywidget = g_object_get_data(G_OBJECT(button), "entrywidget");
    name = (char*)glade_get_widget_name(GTK_WIDGET(entrywidget));

    if (!strcmp(name,"entry_eps_filename") || !strcmp(name,"entry_tray_filename"))
        startdir = (gchar*)gtk_entry_get_text( GTK_ENTRY(lookup_widget(GTK_WIDGET(entrywidget),"ps_entry")) );
    else
        startdir = (gchar*) gtk_entry_get_text( GTK_ENTRY(entrywidget) );

    startdir = g_filename_from_utf8(startdir,strlen(startdir), NULL, NULL, NULL);

    /* To make the file selection box open up *in* the proper directory
     * the dir name should have a "/" at the end. However since this is
     * a general function we dont know whether the entry box contained a
     * dir name or a file name. We should check and if it is a dir name
     * add a "/"*/
    if (strlen(startdir) == 0)
        startdir = g_strdup("/");
    else
      {
        stat(startdir,&statbuf);
        if (S_ISDIR(statbuf.st_mode))
          {
            if (startdir[strlen(startdir)-1] != '/' && strlen(startdir) <= 1000)
                strcat(startdir,"/");
            else if (startdir[strlen(startdir)-1] != '/' && strlen(startdir) > 1000)
              {
                startdir = realloc(startdir,strlen(startdir)+2);
                strcat(startdir,"/");
              }
          }
      }
    show_file_selector(startdir,tmp);
    if (strlen(tmp) != 0) {
        gtk_entry_set_text(GTK_ENTRY(entrywidget),tmp);
    }

}


void
on_get_cddb_clicked                    (GtkButton       *button,
        gpointer         user_data)
{
    int i,status;
    GtkWidget *w;
    GtkTextBuffer *tbuffer;
    GtkTextIter start,end,curr;
    gchar *devicename, *cddbserver;
    cdinfo_t *cdinfo;
    
    /* get the device name */
    w = lookup_widget( GTK_WIDGET(button), "entry_cddb");
    devicename = (gchar*)gtk_entry_get_text( GTK_ENTRY(w) );
    devicename = g_filename_from_utf8(devicename, strlen(devicename), NULL,NULL,NULL);

    status = check_filename(devicename, CHECK_FILE_BLOCK, "Path to CDROM device");
    if (status < 0) return;

    w = lookup_widget( GTK_WIDGET(button), "cddbserver_entry");
    cddbserver  = (gchar*)gtk_entry_get_text( GTK_ENTRY(w) );
    if (strlen(cddbserver) == 0)
        cddbserver = strdup("freedb.freedb.org");

    
    /* get the results of the query */
    cdinfo = cddb_lookup( devicename, cddbserver );
    if (!cdinfo) return; 

    w = lookup_widget( GTK_WIDGET(button), "text1" );
    tbuffer = gtk_text_view_get_buffer( GTK_TEXT_VIEW(w) );
    gtk_text_buffer_set_text( tbuffer, "", 0 );
    
    for (i = 0; i < cdinfo->numtrack; i++)
      {
        char num[6];
        printf("%s\n", cdinfo->tracks[i].title);
        sprintf(num,"%d.\t",i+1);
        gtk_text_buffer_get_bounds( tbuffer, &start, &end );
        gtk_text_buffer_insert(tbuffer, &end, num, strlen(num));
        gtk_text_buffer_get_bounds( tbuffer, &start, &end );
        gtk_text_buffer_insert(tbuffer, &end, cdinfo->tracks[i].title, strlen(cdinfo->tracks[i].title));
        gtk_text_buffer_get_bounds( tbuffer, &start, &end );
        gtk_text_buffer_insert(tbuffer, &end, "\n", 1);
        
      }

    /* set the Category and Subcategory values */
    w = lookup_widget( GTK_WIDGET(button), "entry_kategori" );
    gtk_entry_set_text( GTK_ENTRY(w), cdinfo->albname);
    w = lookup_widget( GTK_WIDGET(button), "entry_underkategory" );
    gtk_entry_set_text( GTK_ENTRY(w), cdinfo->artname);

    //gtk_text_view_set_buffer( GTK_TEXT_VIEW(w), tbuffer );

    /* update the text view */
}


void
on_path_for_to_file_changed            (GtkEditable     *editable,
        gpointer         user_data)
{
    GtkWidget *make_label, *w;
    gchar *filepath;

    w = lookup_widget(GTK_WIDGET(editable),"path_for_to_file");
    filepath = (gchar*) gtk_entry_get_text( GTK_ENTRY(w) );
    filepath = g_filename_from_utf8( filepath, strlen(filepath), NULL, NULL, NULL);
    make_label = lookup_widget( GTK_WIDGET(editable), "make_label");

    if ( strlen(filepath) == 0)
        gtk_widget_set_sensitive(make_label, FALSE);
    else
        gtk_widget_set_sensitive(make_label, TRUE);
}


void
on_make_label_clicked(GtkButton *button, gpointer user_data)
{
    gchar *command_make;
    gchar *path_for_to_file;
    gchar gtkcdlabeltmpfile[MAXLINE];
    int status;

    GtkWidget *entry_path_for_to_file;

    snprintf(gtkcdlabeltmpfile, MAXLINE-1, "%s/gtkcdlabel-tmp-ps", getenv("HOME"));
    entry_path_for_to_file  =       lookup_widget (GTK_WIDGET (button), "path_for_to_file");
    path_for_to_file        =       (gchar*)gtk_entry_get_text( GTK_ENTRY(entry_path_for_to_file) );
    path_for_to_file = g_filename_from_utf8(path_for_to_file, strlen(path_for_to_file), NULL, NULL, NULL);

    if (strlen(path_for_to_file) == 0) return;

    status = check_filename(path_for_to_file, CHECK_FILE_OUTPUT,NULL);
    if (status < 0) return;
    
    status = run_cdlabelgen(button,path_for_to_file);
    if (status < 0) return;

    if (!is_file(gtkcdlabeltmpfile))
        syntaxerror (button);
    else
      {
        command_make = g_malloc( strlen(path_for_to_file) + strlen(gtkcdlabeltmpfile)+ 20);

        if (is_file(path_for_to_file))
          {
            if ( yes_no_dialog(_("File exists"),_("Overwrite file") ))
              {
                sprintf(command_make,"/bin/mv -f %s %s",gtkcdlabeltmpfile , path_for_to_file);
                system(command_make);
              }
          }
        else
          {
            sprintf(command_make, "/bin/mv %s %s",gtkcdlabeltmpfile, path_for_to_file);
            system(command_make);
          }
      }

}


void
on_preview_clicked                     (GtkButton       *button,
        gpointer         user_data)
{
    gchar *where_is_viewer;
    gchar *command_view;
    gchar gtkcdlabeltmpfile[MAXLINE];
    int status;

    GtkWidget *viewer_entry;

    snprintf(gtkcdlabeltmpfile, MAXLINE-1,"%s/gtkcdlabel-tmp-ps", getenv("HOME"));
    viewer_entry            =       lookup_widget (GTK_WIDGET (button), "viewer_entry");
    where_is_viewer         =       g_locale_from_utf8(
            (gchar*)gtk_entry_get_text( GTK_ENTRY(viewer_entry)),
            strlen((gchar*)gtk_entry_get_text( GTK_ENTRY(viewer_entry))),
            NULL,NULL,NULL);


    command_view = g_malloc( strlen(where_is_viewer) + strlen(gtkcdlabeltmpfile) + 10);
    
    /* check that the viewer program is specified as a file */
    status = check_filename(where_is_viewer,CHECK_FILE_VALID,"The path to the preview program");
    if (status < 0) return;

    status = run_cdlabelgen(button,user_data);
    if (status < 0) return;

    if (!is_file(gtkcdlabeltmpfile))
        syntaxerror (button);
    else
      {
        sprintf(command_view,"%s \"%s\"",where_is_viewer,gtkcdlabeltmpfile);
        system(command_view);
        remove(gtkcdlabeltmpfile);
      }

}


void
on_print_clicked                       (GtkButton       *button,
        gpointer         user_data)
{
    gchar *lpr_entry="\0";
    gchar *command_lpr;
    gchar gtkcdlabeltmpfile[MAXLINE];
    GtkWidget *entry_lpr_entry;   
    int status;

    snprintf(gtkcdlabeltmpfile, MAXLINE-1, "%s/gtkcdlabel-tmp-ps", getenv("HOME"));
    entry_lpr_entry      =       lookup_widget (GTK_WIDGET (button), "lpr_entry");
    lpr_entry            =       (gchar*)gtk_entry_get_text( GTK_ENTRY(entry_lpr_entry) );
    lpr_entry = g_filename_from_utf8(lpr_entry, strlen(lpr_entry), NULL, NULL, NULL);

    if ( strlen(lpr_entry)   == 0){ strcpy(lpr_entry, "lpr");}
    command_lpr = g_malloc( strlen(lpr_entry) + strlen(gtkcdlabeltmpfile) + 10);

    status = check_filename(lpr_entry,CHECK_FILE_VALID,"The path to the printing program");
    if (status < 0) return;
        
    status = run_cdlabelgen(button,user_data);
    
    if (status < 0) return;

    if (!is_file(gtkcdlabeltmpfile))
      {
        syntaxerror (button);
      }
    else
      {
        sprintf(command_lpr,"%s \"%s\"", lpr_entry, gtkcdlabeltmpfile);
        system(command_lpr);
        remove(gtkcdlabeltmpfile);
      }

}


gboolean
on_app1_delete_event(GtkWidget       *widget,
        GdkEvent        *event,
        gpointer         user_data)
{
    GtkWidget *w,*b;

    w = lookup_widget( widget, "save_on_exit");
    if ( GTK_TOGGLE_BUTTON(w)->active )
      {
        b = lookup_widget( widget, "save_settings");
        on_save_settings_clicked (GTK_BUTTON(b), NULL);
      }

    gtk_main_quit();
    return FALSE;
}







void
on_get_item_file_clicked               (GtkButton       *button,
        gpointer         user_data)
{
    file_to_item_text(button);
}


void
on_save_settings_clicked               (GtkButton       *button,
        gpointer         user_data)
{
    FILE *fd;
    gchar tmp[MAXLINE];
    gchar tmp2[MAXLINE];

    strcpy(tmp2, g_get_home_dir());
    strcat(tmp2, "/.gtkcdlabelrc");

    fd = fopen(tmp2,"w");
    if (fd == NULL) {
        /* error opening file */
        g_print(_("Could not open config file - aborting!"));
        return;
    }

    fputs("# gtkcdlabel Configuration-File\n",fd);
    fputs("#\n",fd);
    fputs("# Don't edit! (Unless you REALLY know what you are doing)\n",fd);
    fputs("#\n\n",fd);

    g_snprintf(tmp,MAXLINE,"cdlabelgen_entry = \"%s\"\n",
            gtk_entry_get_text( GTK_ENTRY(lookup_widget (GTK_WIDGET (button), "cdlabelgen_entry")) ) );
    fputs(tmp,fd);

    g_snprintf(tmp,MAXLINE,"viewer_entry = \"%s\"\n",
            gtk_entry_get_text( GTK_ENTRY(lookup_widget (GTK_WIDGET (button), "viewer_entry")) ) );
    fputs(tmp,fd);

    g_snprintf(tmp,MAXLINE,"lpr_entry = \"%s\"\n",
            gtk_entry_get_text( GTK_ENTRY(lookup_widget (GTK_WIDGET (button), "lpr_entry")) ) );
    fputs(tmp,fd);

    g_snprintf(tmp,MAXLINE,"ps_entry = \"%s\"\n",
            gtk_entry_get_text(GTK_ENTRY(lookup_widget (GTK_WIDGET (button), "ps_entry")) ) );
    fputs(tmp,fd);

    g_snprintf(tmp,MAXLINE,"entry_cddb = \"%s\"\n",
            gtk_entry_get_text(GTK_ENTRY(lookup_widget (GTK_WIDGET (button), "entry_cddb")) ) );
    fputs(tmp,fd);

    g_snprintf(tmp,MAXLINE,"template_entry = \"%s\"\n",
            gtk_entry_get_text(GTK_ENTRY(lookup_widget (GTK_WIDGET (button), "template_entry")) ) );
    fputs(tmp,fd);
    
    g_snprintf(tmp,MAXLINE,"mountpointentry = \"%s\"\n",
            gtk_entry_get_text(GTK_ENTRY(lookup_widget (GTK_WIDGET (button), "mountpointentry")) ) );
    fputs(tmp,fd);
    
    g_snprintf(tmp,MAXLINE,"cddbserver_entry = \"%s\"\n",
            gtk_entry_get_text(GTK_ENTRY(lookup_widget (GTK_WIDGET (button), "cddbserver_entry")) ) );
    fputs(tmp,fd);
    
    g_snprintf(tmp,MAXLINE,"convert_entry = \"%s\"\n",
            gtk_entry_get_text(GTK_ENTRY(lookup_widget (GTK_WIDGET (button), "convert_entry")) ) );
    fputs(tmp,fd);
    
    g_snprintf(tmp,MAXLINE,"save_on_exit = \"%i\"\n",
            GTK_TOGGLE_BUTTON(lookup_widget (GTK_WIDGET (button), "save_on_exit"))->active );
    fputs(tmp,fd);

    /****************************************************************************************
     *
     * Save the color values
     */
    g_snprintf(tmp,MAXLINE,"spinbutton1 = \"%d\"\n", gtk_spin_button_get_value_as_int(
                GTK_SPIN_BUTTON(lookup_widget(GTK_WIDGET(button),"spinbutton1"))));
    fputs(tmp,fd);

    g_snprintf(tmp,MAXLINE,"spinbutton3 = \"%d\"\n", gtk_spin_button_get_value_as_int(
                GTK_SPIN_BUTTON(lookup_widget(GTK_WIDGET(button),"spinbutton3"))));
    fputs(tmp,fd);

    g_snprintf(tmp,MAXLINE,"spinbutton4 = \"%d\"\n", gtk_spin_button_get_value_as_int(
                GTK_SPIN_BUTTON(lookup_widget(GTK_WIDGET(button),"spinbutton4"))));
    fputs(tmp,fd);

    g_snprintf(tmp,MAXLINE,"spinbutton5 = \"%d\"\n", gtk_spin_button_get_value_as_int(
                GTK_SPIN_BUTTON(lookup_widget(GTK_WIDGET(button),"spinbutton5"))));
    fputs(tmp,fd);

    g_snprintf(tmp,MAXLINE,"spinbutton6 = \"%d\"\n", gtk_spin_button_get_value_as_int(
                GTK_SPIN_BUTTON(lookup_widget(GTK_WIDGET(button),"spinbutton6"))));
    fputs(tmp,fd);

    g_snprintf(tmp,MAXLINE,"spinbutton7 = \"%d\"\n", gtk_spin_button_get_value_as_int(
                GTK_SPIN_BUTTON(lookup_widget(GTK_WIDGET(button),"spinbutton7"))));
    fputs(tmp,fd);

    g_snprintf(tmp,MAXLINE,"spinbutton8 = \"%d\"\n", gtk_spin_button_get_value_as_int(
                GTK_SPIN_BUTTON(lookup_widget(GTK_WIDGET(button),"spinbutton8"))));
    fputs(tmp,fd);

    g_snprintf(tmp,MAXLINE,"spinbutton9 = \"%d\"\n", gtk_spin_button_get_value_as_int(
                GTK_SPIN_BUTTON(lookup_widget(GTK_WIDGET(button),"spinbutton9"))));
    fputs(tmp,fd);

    g_snprintf(tmp,MAXLINE,"spinbutton10 = \"%d\"\n", gtk_spin_button_get_value_as_int(
                GTK_SPIN_BUTTON(lookup_widget(GTK_WIDGET(button),"spinbutton10"))));
    fputs(tmp,fd);

    g_snprintf(tmp,MAXLINE,"spinbutton11 = \"%d\"\n", gtk_spin_button_get_value_as_int(
                GTK_SPIN_BUTTON(lookup_widget(GTK_WIDGET(button),"spinbutton11"))));
    fputs(tmp,fd);

    g_snprintf(tmp,MAXLINE,"spinbutton12 = \"%d\"\n", gtk_spin_button_get_value_as_int(
                GTK_SPIN_BUTTON(lookup_widget(GTK_WIDGET(button),"spinbutton12"))));
    fputs(tmp,fd);

    g_snprintf(tmp,MAXLINE,"spinbutton13 = \"%d\"\n", gtk_spin_button_get_value_as_int(
                GTK_SPIN_BUTTON(lookup_widget(GTK_WIDGET(button),"spinbutton13"))));
    fputs(tmp,fd);

    /* Save the linewidth */
    g_snprintf(tmp,MAXLINE,"linewidth_sp = \"%f\"\n", gtk_spin_button_get_value_as_float(
                GTK_SPIN_BUTTON(lookup_widget(GTK_WIDGET(button),"linewidth_sp"))));
    fputs(tmp,fd);

    /* Save the papertype option */
    g_snprintf(tmp,MAXLINE,"papertype_optionmenu = \"%d\"\n", gtk_option_menu_get_history(
                GTK_OPTION_MENU(lookup_widget(GTK_WIDGET(button),"papertype_optionmenu"))));
    fputs(tmp,fd);


    if (fclose(fd) != 0) 
      {
        /* error closing file */
        g_print(_("Could not save config file - aborting!"));
        return;
      }
}


void
on_clear_textfield_clicked             (GtkButton       *button,
        gpointer         user_data)
{
    GtkWidget *text1;
    GtkTextBuffer *buffer;

    text1 = lookup_widget(GTK_WIDGET(button), "text1");
    buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text1));
    gtk_text_buffer_set_text(GTK_TEXT_BUFFER(buffer),"",-1);
}


void
on_get_to_file_clicked(GtkButton *button, gpointer user_data)
{
    char tmp[MAXLINE];
    show_file_selector(NULL,tmp);
    if (strlen(tmp) != 0) {
        gtk_entry_set_text(GTK_ENTRY(lookup_widget(GTK_WIDGET(button),"path_for_to_file")),tmp);
    }
}


void
on_closebutton1_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *dlg = lookup_widget(GTK_WIDGET(button), "error_dialog");
    gtk_widget_destroy( GTK_WIDGET(dlg) );
    return;
}



void on_ced_button_clicked (GtkButton *button, gpointer user_data)
{
    GtkWidget *w = lookup_widget( GTK_WIDGET(button), "cdlabelgen_error");
    gtk_widget_destroy(w);
}

void on_ornamentbutton_clicked (GtkButton *button, gpointer user_data)
{
    gboolean s;
    GtkWidget *w,*w1;
    char *name, *ornament;
    GtkTextBuffer *tbuffer;
    GtkTextIter iter;
    GtkTextMark *mark, *curmark;

    name = (gchar*)glade_get_widget_name( GTK_WIDGET(button) );
    if (!strcmp("italicbutton", name))
        ornament = strdup("{#I}");
    else if (!strcmp("boldbutton", name))
        ornament = strdup("{#B}");
    else if (!strcmp("italicboldbutton",name))
        ornament = strdup("{#BI}");

    w = lookup_widget( GTK_WIDGET(button), "text1" );
    tbuffer = gtk_text_view_get_buffer( GTK_TEXT_VIEW(w) );
    
    /* get an iter for the current position via the 'insert' mark */
    mark = gtk_text_buffer_get_insert(tbuffer);
    gtk_text_buffer_get_iter_at_mark(tbuffer, &iter, mark);

    /* set a mark at the current cursor pos */
    curmark = gtk_text_buffer_create_mark(tbuffer, "curpos", &iter, FALSE);

    gtk_text_view_backward_display_line_start(GTK_TEXT_VIEW(w), &iter);
    gtk_text_buffer_insert(tbuffer, &iter, ornament, strlen(ornament));
        
    /* use the saved mark to reset the cursor to wherever it was before
     * the user inserted an ornament tag */
    gtk_text_buffer_get_iter_at_mark(tbuffer, &iter, curmark);
    gtk_text_buffer_place_cursor(tbuffer, &iter); 
    gtk_text_buffer_delete_mark(tbuffer, curmark);

    /* also set the focus back to the text view */
    w1 = lookup_widget( GTK_WIDGET(button), "window1" );
    gtk_window_set_focus( GTK_WINDOW(w1), GTK_WIDGET(w) );

    return;
}

/*********************************************************************/


void on_mp3button_clicked (GtkButton *button, gpointer user_data)
{
    int i,j,status,nomatch = 0;
    char *olddir,*errstr;
    GtkWidget *w;
    GladeXML *xml;
    GtkTextBuffer *tbuffer;
    gchar *mnt;
    glob_t g;
    int numdir;
    GArray *dirlist,*id3error;
    struct id3info tag;

    w = lookup_widget( GTK_WIDGET(button), "mountpointentry" );
    mnt = (gchar*)gtk_entry_get_text( GTK_ENTRY(w) );
    mnt = g_filename_from_utf8(mnt,strlen(mnt),NULL,NULL,NULL);

    olddir = getcwd(NULL,0);

    /* try and change to the mount point */
    status = chdir(mnt);
    if (status == -1)
      {
        char *errstr = strdup(strerror(errno));
        char *e = malloc( sizeof(char) * 
                ( strlen(errstr) + 
                  strlen(mnt) +
                  strlen("Could'nt change to the CDROM mount point: : ") +
                  5 ));
        sprintf(e,"%s%s: %s", 
                "Could'nt change to the CDROM mount point: ", mnt, errstr);
                  
        xml = create_error_dialog();
        w = glade_xml_get_widget(xml, "error_dialog_label");
        gtk_label_set_text(GTK_LABEL(w), e);
        free(e);
        return;
      }
    
    /* Get a list of directories under the mnt point */
    dirlist = g_array_new(FALSE,FALSE,sizeof(char*));
    numdir = 0;
    getdirs(getcwd(NULL,0), &numdir, &dirlist);
    g_array_append_val(dirlist, mnt);
    numdir++;



    id3error = g_array_new(FALSE, FALSE, sizeof(char*));

    /* Now loop through each directory - the dir's will be sorted
     * alphabetically, so all we need to do is to sort the file names
     * under each directory */
    for ( i = 0; i < numdir; i++)
      {
        char *tmp;
        char *path1; 
        char *path2;
        char *d;

        d = g_array_index(dirlist,char*,i);
        
        path1 = (char*) malloc(sizeof(char) * (strlen(d) + strlen("/*.mp3") + 4));
        path2  = (char*) malloc(sizeof(char) * (strlen(d) + strlen("/*.MP3") + 4));
        sprintf(path1,"%s/*.mp3",d);
        sprintf(path2,"%s/*.MP3",d);

        /* now try and get a list of files under this directory */
        status = glob(path1, GLOB_ERR, NULL, &g);
        if (status != 0)
          {
            switch(status)
              {
                case GLOB_ABORTED:
                    break;
                case GLOB_NOSPACE:
                    errstr = strdup("Out of memory!");
                    break;
                case GLOB_NOMATCH:
                    status = glob(path2, GLOB_APPEND , NULL, &g);
                    if (status != 0)
                      {
                        switch(status)
                          {
                            case GLOB_NOMATCH:
                                nomatch++;
                                break;
                          }
                      }
                    break;
              }
            continue;
          }


        /* Add the directory name */
        w = lookup_widget( GTK_WIDGET(button), "text1" );
        tbuffer = gtk_text_view_get_buffer( GTK_TEXT_VIEW(w) );
        gtk_text_buffer_insert_at_cursor(tbuffer,"{#B}",strlen("{#B}"));

        gtk_text_buffer_insert_at_cursor(
                tbuffer,
                (char*)my_basename(d), 
                g_utf8_strlen((char*)my_basename(d),-1));
        gtk_text_buffer_insert_at_cursor(tbuffer,"\n",strlen("\n"));


        /* for each file get the mp3 tags and add them to the text view*/
        for (j = 0; j < g.gl_pathc; j++)
          {
            char *txt;

            status = get_id3tag(g.gl_pathv[j], &tag);
            switch(status)
              {
                case -2:
                    if (g_utf8_validate(g.gl_pathv[j],-1,NULL))
                      {
                        txt = strdup(g.gl_pathv[j]);
                        g_array_append_val(id3error, txt);
                      }
                    else
                      {
                        txt = g_filename_to_utf8(
                                g.gl_pathv[j],
                                strlen(g.gl_pathv[j]),
                                NULL,NULL,NULL);
                        g_array_append_val(id3error, txt);
                      }
                    continue;
                case -1:
                    txt = malloc( sizeof(char) * ( strlen(g.gl_pathv[j]) + 20));
                    sprintf(txt,"{#I}%d.\t%s\n",j+1,my_basename(g.gl_pathv[j]));
                    break;
                case 0:
                    if (strlen(tag.artist) == 0)
                      {
                        txt = malloc( sizeof(char) * ( strlen(tag.title) + 20));
                        sprintf(txt,"{#I}%d.\t%s\n",j+1,tag.title);
                      }
                    else
                      {
                        txt = malloc( sizeof(char) * ( strlen(tag.title) + strlen(tag.artist) + 15));
                        sprintf(txt,"{#I}%d.\t%s- %s\n",j+1,tag.artist,tag.title);
                      }
                    break;
              }

            gtk_text_buffer_insert_at_cursor(tbuffer,txt, strlen(txt));

            free(txt);
          }
        globfree(&g);
        gtk_text_buffer_insert_at_cursor(tbuffer,"\n",strlen("\n"));

        free(path1);
        free(path2);


      }

    /* if id3error has any elements they are the filenames of files
     * that did'nt have ID3 tags. Show an error box with a list of
     * them */
    if (id3error->len > 0)
      {
        GladeXML *xml;
        GtkWidget *w;
        GtkTextBuffer *tbuffer;
        int k;

        xml = create_cdlabelgen_error();
        w = glade_xml_get_widget(xml, "cdlabelgen_error");
        gtk_window_set_title( GTK_WINDOW(w), "ID3 Errors");

        w = glade_xml_get_widget(xml, "ced_textview");
        tbuffer = gtk_text_view_get_buffer( GTK_TEXT_VIEW(w) );
        gtk_text_buffer_insert_at_cursor( tbuffer, 
                "The following files did not have any ID3 tags:\n\n",
                strlen("The following files did not have any ID3 tags:\n\n"));
        
        for (k = 0; k < id3error->len; k++)
          {
            char *f = g_array_index(id3error, char*,k);
            //printf("%s %d\n", f,k);

            gtk_text_buffer_insert_at_cursor(tbuffer,
                    g_array_index(id3error, char*,k),
                    strlen(g_array_index(id3error, char*,k)));
            gtk_text_buffer_insert_at_cursor(tbuffer,"\n", strlen("\n"));
          }
        
        //gtk_text_buffer_set_text(tbuffer, error_buffer, strlen(error_buffer));

      }

    if (nomatch == numdir)
      {
        errstr = strdup("Could'nt find any mp3 files. Are you sure the CD is mounted?");
        xml = create_error_dialog();
        w = glade_xml_get_widget(xml, "error_dialog_label");
        gtk_label_set_text(GTK_LABEL(w), errstr);
      }


    /* make sure to go back to the original directory */
    g_array_free(dirlist,TRUE);
    g_array_free(id3error,TRUE);

    chdir(olddir);
    free(olddir);
    return;
}

/*********************************************************************************
 *
 * These callbacks make certain widgets sensitive/insensitive 
 *
 */
void on_use_eps_toggled(GtkToggleButton *button, gpointer data)
{
    GtkWidget *w;
    gboolean toggled;
    char *names[] = {
        "eps_scale",
        "entry_eps_filename",
        "eps_select_file",
        "cover_bg_choice",
        "coverxoffset",
        "coveryoffset"
    };

    toggled = gtk_toggle_button_get_active(button);

    if (toggled)
      {
        sensitize(GTK_WIDGET(button), 6, names, TRUE);

        /* If use_cover_items was clicked OFF when the cover_bg_choice was
         * ON then when use_cover_items is clicked ON the X & Y offset
         * entryies will be sensitive - but they should'nt be since
         * cover_bg_choice will be still ON. So we need to check for that */
        w = lookup_widget( GTK_WIDGET(button), "cover_bg_choice");
        on_cover_bg_choice_toggled(GTK_TOGGLE_BUTTON(w), NULL);
      }
    else
        sensitize(GTK_WIDGET(button), 6, names, FALSE);

}

void on_use_cover_items_toggled(GtkToggleButton *button, gpointer data)
{
    gboolean toggled;
    char *names[] = {
        "entry_cover_items"
    };

    toggled = gtk_toggle_button_get_active(button);

    if (toggled)
        sensitize(GTK_WIDGET(button), 1, names, TRUE);
    else
        sensitize(GTK_WIDGET(button), 1, names, FALSE);

}

void on_use_tray_toggled(GtkToggleButton *button, gpointer data)
{
    GtkWidget *w;
    gboolean toggled;
    char *names[] = {
        "entry_tray_filename",
        "tray_scale",
        "tray_select_file",
        "trayxoffset",
        "trayyoffset",
        "tray_bg_choice",
        "tray_bg_option"

    };

    toggled = gtk_toggle_button_get_active(button);

    if (toggled)
      {
        sensitize(GTK_WIDGET(button), 7, names, TRUE);

        w = lookup_widget(GTK_WIDGET(button),"tray_bg_choice");
        on_tray_bg_choice_toggled(GTK_TOGGLE_BUTTON(w),NULL);
      }
    else
        sensitize(GTK_WIDGET(button), 7, names, FALSE);

}

void on_tray_bg_choice_toggled(GtkToggleButton *button, gpointer data)
{
    GtkWidget *w;
    gboolean toggled;
    char *names[] = {
        "trayxoffset",
        "trayyoffset"
    };
    
    toggled = gtk_toggle_button_get_active(button);
    w = lookup_widget(GTK_WIDGET(button), "tray_bg_option");

    if (toggled)
      {
        sensitize(GTK_WIDGET(button), 2, names, FALSE);
        gtk_widget_set_sensitive(w, TRUE);
      }
    else
      {
        sensitize(GTK_WIDGET(button), 2, names, TRUE);
        gtk_widget_set_sensitive(w, FALSE);
      }
}
    
void on_cover_bg_choice_toggled(GtkToggleButton *button, gpointer data)
{
    gboolean toggled;
    char *names[] = {
        "coverxoffset",
        "coveryoffset"
    };

    toggled = gtk_toggle_button_get_active(button);

    if (toggled)
        sensitize(GTK_WIDGET(button), 2, names, FALSE);
    else
        sensitize(GTK_WIDGET(button), 2, names, TRUE);
}



    
